@@ -601,7 +601,7 @@ expand_line (char *buffer)
       if (!value)
         value = "";
       valuelen = strlen (value);
-      if (valuelen <= pend - p)
+      if (valuelen <= (size_t) (pend - p))
         {
           memcpy (p, value, valuelen);
           p += valuelen;
@@ -657,6 +657,7 @@ cmd_let (const char *assign_to, char *arg)
 static void
 cmd_echo (const char *assign_to, char *arg)
 {
+  (void)assign_to;
   if (!opt_no_echo)
     printf ("%s\n", arg);
 }
@@ -664,6 +665,7 @@ cmd_echo (const char *assign_to, char *arg)
 static void
 cmd_send (const char *assign_to, char *arg)
 {
+  (void)assign_to;
   if (opt_verbose)
     fprintf (stderr, "sending `%s'\n", arg);
   write_assuan (server_send_fd, arg); 
@@ -690,6 +692,8 @@ handle_status_line (char *arg)
 static void
 cmd_expect_ok (const char *assign_to, char *arg)
 {
+  (void)assign_to;
+  (void)arg;
   if (opt_verbose)
     fprintf (stderr, "expecting OK\n");
   do
@@ -708,6 +712,8 @@ cmd_expect_ok (const char *assign_to, char *arg)
 static void
 cmd_expect_err (const char *assign_to, char *arg)
 {
+  (void)assign_to;
+  (void)arg;
   if (opt_verbose)
     fprintf (stderr, "expecting ERR\n");
   do
@@ -779,6 +785,7 @@ cmd_createfile (const char *assign_to, char *arg)
 static void
 cmd_pipeserver (const char *assign_to, char *arg)
 {
+  (void)assign_to;
   if (!*arg)
     die ("syntax error: servername missing");
 
@@ -789,6 +796,7 @@ cmd_pipeserver (const char *assign_to, char *arg)
 static void
 cmd_quit_if(const char *assign_to, char *arg)
 {
+  (void)assign_to;
   if (eval_boolean (arg))
     exit (0);
 }
@@ -796,6 +804,7 @@ cmd_quit_if(const char *assign_to, char *arg)
 static void
 cmd_fail_if(const char *assign_to, char *arg)
 {
+  (void)assign_to;
   if (eval_boolean (arg))
     exit (1);
 }
@@ -903,7 +912,7 @@ interpreter (char *line)
     { "fail-if"   , cmd_fail_if },
     { "cmpfiles"  , cmd_cmpfiles },
     { "getenv"    , cmd_getenv },
-    { NULL }
+    { NULL, NULL }
   };
   char *p, *save_p;
   int i, save_c;
